using System;
using EnvDTE;
using EnvDTE80;
using VSSDKAssist.Assets.Helpers;
using VSSDKAssist.Assets.ValueProviders.Base;

namespace VSSDKAssist.Recipes.Common.ValueProviders
{
	/// <summary>
	/// Value provider for c# attribute value
	/// </summary>
	public class AttributeValueValueProvider : ValueProviderBase
	{
		#region Constants
		private const string CodeClassArgument = "CodeClassArgument";
		private const string AttributeName = "AttributeName";
		private const string AttributeArgument = "AttributeArgument";
		#endregion

		#region Fields
		private string codeClassArgument;
		private string attributeName;
		private string attributeArgument;
		#endregion

		/// <summary>
		/// Contains code that will be called when recipe execution begins. This is the first method in the lifecycle.
		/// </summary>
		/// <param name="currentValue">An <see cref="T:System.Object"/> that contains the current value of the argument.</param>
		/// <param name="newValue">When this method returns, contains an <see cref="T:System.Object"/> that contains
		/// the new value of the argument, if the returned value
		/// of the method is <see langword="true"/>. Otherwise, it is ignored.</param>
		/// <returns>
		/// 	<see langword="true"/> if the argument value should be replaced with
		/// the value in <paramref name="newValue"/>; otherwise, <see langword="false"/>.
		/// </returns>
		/// <remarks>By default, always returns <see langword="false"/>, unless overriden by a derived class.</remarks>
		public override bool OnBeginRecipe(object currentValue, out object newValue)
		{
			newValue = null;

			if((currentValue == null) && (this.ExecutionStage != Stage.OnBeforeActions))
			{
				CodeClass codeClassArgumentValue = DictionaryService.GetValue(codeClassArgument) as CodeClass;

				if(codeClassArgumentValue != null)
				{
					CodeAttribute attribute = FileCodeModelHelper.GetAttribute(codeClassArgumentValue, attributeName);

					if(attribute != null)
					{
						if(!string.IsNullOrEmpty(this.attributeArgument))
						{
							CodeAttributeArgument argument = FileCodeModelHelper.GetAttributeArgument(attribute, this.attributeArgument);

							if(argument != null)
							{
								newValue = SanitizeValue(attribute.Value);
								return true;
							}
						}
						else
						{
							newValue = SanitizeValue(attribute.Value);
							return true;
						}
					}
				}
			}

			return false;
		}

		/// <summary>
		/// Called when [configure].
		/// </summary>
		/// <param name="attributes">The attributes.</param>
		protected override void OnConfigure(System.Collections.Specialized.StringDictionary attributes)
		{
			if(attributes == null)
				throw new ArgumentNullException("attributes");

			if(attributes.ContainsKey(CodeClassArgument))
			{
				codeClassArgument = attributes[CodeClassArgument];
			}

			if(attributes.ContainsKey(AttributeName))
			{
				attributeName = attributes[AttributeName];
			}

			if(attributes.ContainsKey(AttributeArgument))
			{
				attributeArgument = attributes[AttributeArgument];
			}
		}

		#region Private Implementation
		private static string SanitizeValue(string attValue)
		{
			string formattedValue = attValue;
			formattedValue = attValue.Replace("@\"", "");
			formattedValue = formattedValue.Replace("\"", "");
			formattedValue = formattedValue.Replace("\\\\", "\\");

			return formattedValue;
		}
		#endregion
	}
}